@@ -2,15 +2,15 @@
 """
 
 import re
-import time
 import argparse
 import subprocess
-from beartype import beartype
-from datetime import datetime
 from pathlib import Path
+from datetime import datetime
 from itertools import zip_longest
+
+from beartype import beartype
+from beartype.cave import NoneTypeOr
 from watchlogs.watchlogs import Watcher
-from typing import Optional
 
 
 class Yaspi:
@@ -22,8 +22,8 @@ class Yaspi:
             cmd: str,
             prep: str,
             recipe: str,
-            gen_script_dir: Path,
-            log_dir: Path,
+            gen_script_dir: str,
+            log_dir: str,
             partition: str,
             job_array_size: int,
             cpus_per_task: int,
@@ -36,9 +36,10 @@ class Yaspi:
             throttle_array: int,
             mem: str,
             constraint_str: str,
+            custom_directives: str = "",
             template_dir: Path = Path(__file__).parent / "templates",
-            job_queue: Optional[str] = None,
-            env_setup: Optional[str] = None,
+            job_queue: NoneTypeOr[str] = None,
+            env_setup: NoneTypeOr[str] = None,
     ):
         self.cmd = cmd
         self.mem = mem
@@ -57,16 +58,17 @@ class Yaspi:
         self.gpus_per_task = gpus_per_task
         self.constraint_str = constraint_str
         self.throttle_array = throttle_array
-        self.gen_script_dir = gen_script_dir
+        self.gen_script_dir = Path(gen_script_dir)
         self.job_array_size = job_array_size
         self.use_custom_ray_tmp_dir = use_custom_ray_tmp_dir
+        self.custom_directives = custom_directives
         self.slurm_logs = None
         # SLURM expects the logfiles to be absolute paths
         self.log_dir = Path(log_dir).resolve()
         self.generate_scripts()
 
     def generate_scripts(self):
-        gen_dir = Path(self.gen_script_dir)
+        gen_dir = self.gen_script_dir
         if self.env_setup is None:
             self.env_setup = (
                 'export PYTHONPATH="${BASE}":$PYTHONPATH\n'
@@ -166,6 +168,7 @@ class Yaspi:
                     "cpus_per_task": self.cpus_per_task,
                     "exclude_nodes": f"#SBATCH --exclude={self.exclude}",
                     "sbatch_resources": "",
+                    "custom_directives": self.custom_directives,
                 },
             }
             resource_strs = []
@@ -329,7 +332,7 @@ def main():
     parser.add_argument("--ssh_forward",
                         default="ssh -N -f -R 8080:localhost:8080 triton.robots.ox.ac.uk",
                         help="setup string for a custom environment")
-    parser.add_argument("--log_dir", default="data/slurm-logs", type=Path,
+    parser.add_argument("--log_dir", default="data/slurm-logs", type=str,
                         help="location where SLURM logs will be stored")
     parser.add_argument("--use_custom_ray_tmp_dir", action="store_true")
     parser.add_argument("--refresh_logs", action="store_true")
@@ -341,6 +344,9 @@ def main():
                         help="SLURM --constraint string")
     parser.add_argument("--job_queue", default="",
                         help="a queue of jobs to pass to a yaspi recipe")
+    parser.add_argument("--custom_directives", default="",
+                        help=('Add any extra directives here, separated by newlines'
+                              'e.g. "#SBATCH -A account-name\n#SBATCH --mem 10G"'))
     args = parser.parse_args()
 
     if args.install_location:
@@ -350,7 +356,7 @@ def main():
     # Certain properties use defaults set by the Yaspi class, rather than argparse, to
     # ensure that users of the Python interface (i.e. directly creating Yaspi object)
     # can aslo benefit from these defaults
-    prop_keys = {"template_dir"}
+    prop_keys = {"template_dir", "custom_directives"}
     prop_kwargs = {key: getattr(args, key) for key in prop_keys if getattr(args, key)}
 
     job = Yaspi(
@@ -378,5 +384,6 @@ def main():
     )
     job.submit(watch=bool(args.watch))
 
+
 if __name__ == "__main__":
     main()
